#include <Servo.h> // include the servo

Servo sensServo; 

int servoPin = 5; // define digitalpin 5 for the servo

int motor1 = 11; // define the pins for the H-bridge
int motor1rev = 10;
int motor2 = 6;
int motor2rev = 9;

int sensor = 0; // define analog pin 0 for the sensor

int led = 8; // define the LED on digitalpin 8
int Sensval; // the value from the sensor

int turnDelay = 600; // how long time to turn
const int stopDist = 250; // the dist to stop the robot
int servoDelay = 700; // stop the servo so the sensor can read


void setup()
{
  pinMode(motor1, OUTPUT); // set the motors as an output
  pinMode(motor1rev, OUTPUT);
  pinMode(motor2, OUTPUT);
  pinMode(motor2rev, OUTPUT);
  
  pinMode(led, OUTPUT); // set the LED as an output
  
  pinMode(sensor, INPUT); // set the sensor as an input
  
  Serial.begin(9600); // open the serial monitor
  
  sensServo.attach(servoPin); // attach the servo
  sensServo.write(90); // servo goes to the center
  
  Serial.println("start"); // write start
}

void loop() // void main
{
  drive();
  beginScan();
}

void stay() // robot stop
{
  digitalWrite(led, HIGH); // LED on
  digitalWrite(motor1, LOW);
  digitalWrite(motor1rev, LOW);
  digitalWrite(motor2, LOW);
  digitalWrite(motor2rev, LOW);
  delay(500);
  return;
}


void drive() // go forwards
{
  digitalWrite(led, LOW); // LED off
  analogWrite(motor1, 250);
  digitalWrite(motor1rev, LOW);
  digitalWrite(motor2, HIGH);
  digitalWrite(motor2rev, LOW);
  Serial.println(Sensval);
  return;
}

void backup(int time) // go backwards
{
  digitalWrite(led, LOW); // LED off
  digitalWrite(motor1, LOW);
  digitalWrite(motor1rev, HIGH);
  digitalWrite(motor2, LOW);
  digitalWrite(motor2rev, HIGH);
  delay(time); // delay for the turn
}

void turnL(int time) // turn left
{
  digitalWrite(led, LOW); // LED off
  digitalWrite(motor1, LOW);
  digitalWrite(motor1rev, HIGH);
  digitalWrite(motor2, HIGH);
  digitalWrite(motor2rev, LOW);
  delay(time); // how long the robot will turn
  sensServo.write(95); // servo goes to the center
}

void turnR(int time) // turn right
{
  digitalWrite(led, LOW); // LED off
  digitalWrite(motor1, HIGH);
  digitalWrite(motor1rev, LOW);
  digitalWrite(motor2, LOW);
  digitalWrite(motor2rev, HIGH);
  delay(time); // how long the robot will turn
  sensServo.write(95);
}



void servoL() { // turn the servo to the left

  for(int pos = 90; pos>=30; pos-=1)
  {
    sensServo.write(pos);
    delay(4);
  }
}

void servoR() { // turn the servo to the right

  for(int pos = 90; pos < 155; pos += 1)  
  {
    sensServo.write(pos);
    delay(4);
  }
}

void look() // what the robot will do if it comes to a wall
{
  stay(); // stop the robot
  
  servoL(); // turn the servo to left
  delay(servoDelay); // wait a bit
  int leftDist = analogRead(sensor); // read the distance from the wall on the left
  Serial.println(leftDist);
  Serial.print("   Left Dist");
  sensServo.write(90); // servo goes to the center
  
  servoR(); // turn the servo to right
  delay(servoDelay); // wait a bit
  int rightDist = analogRead(sensor); // read the distance from the wall on the right
  Serial.println(rightDist);
  Serial.print("   Right Dist");
  sensServo.write(90); // servo goes to center
  stay();
  
  if(leftDist < rightDist) // if left value is lower
  {
    turnL(turnDelay); // then turn to left
    Serial.println("Left");
  }
  
  else 
  {
    turnR(turnDelay); // else go to the right
    Serial.println("Right");
  }
  return;
}

void beginRead() // begin to read the distance
{
  drive();
  servoL(); // servo goes to the left
  delay(100); // waits for 100ms
  int leftScan = analogRead(sensor); // read the distance on the left
  
  if(leftScan > stopDist) // if the robot is to close to left
  {
    turnR(300); // the turn to the right for 300ms
  }
}


else // otherwise continue to read 
{
  readC(); 
}
}

void readC() // read the distance in frount
{
  drive();
  sensServo.write(90); // servo goes to the center
  delay(200); // waits for 200ms
  int centerRead = analogRead(sensor); // read the distance
  
  if(centerRead > stopDist) // if the robot is to close
  {
    look(); // start looking for the best way
  }
  
  else // otherwise continue to read
  {
    readR();
  }
}

void readR() // read the dist on the right
{
  drive();
  servoR(); // servo goes to the right
  delay(100); // waits for 100ms
  int rightRead = analogRead(sensor); // read the distance
  
  if(rightRead > stopDist) // if the robot is to close to the right
  {
    turnL(300); // turn left for 300ms
  }
  
  else // otherwise continue to read
  {
    readC2();
  }
}

void readC2() // read the distance in the frount again
{
  drive();
  sensServo.write(90); // servo goes to the center
  delay(200); // waits for 200ms
  int centerRead2 = analogRead(sensor); // read the distance
  
  if(centerRead2 > stopDist) // if the robot is to close
  {
    look(); // start looking for the best way
  }
  
  else // this time it will go back to the left and starts again
  {
    beginRead();
  }
}

  


